#include <windows.h>
#include <stdio.h>
#include "resource.h"

#define WinClassName "systray_class"

HWND ParenthWnd, hWnd;
HMENU hMenu;
NOTIFYICONDATA NID;

BOOL Hidden = false;
BOOL Minimized = false;

LRESULT CALLBACK ParenWindProc(HWND hWnd, UINT MSG, WPARAM wParam, LPARAM lParam)
{
 PAINTSTRUCT PS;
 switch(MSG)
 {
  case WM_PAINT://Lets get the dc when we have to redraw our window
  {
   BeginPaint(hWnd, &PS);
   HDC dc = PS.hdc;
   char str[256] = "Here you can do your usual stuff...";
   TextOut(dc, 0, 0, str, strlen(str));
   EndPaint(hWnd, &PS);
  }break;
  case WM_COMMAND:
  {
   switch(wParam)
   {
    case ID_MENU_TEST: MessageBox(hWnd, "testing", "SysTrayIcon[QSoft]", MB_OK); break;
    case ID_MENU_EXIT: PostQuitMessage(0); break;
   }
  }break;
  case WM_SIZE:
  {
   if (wParam == SIZE_MINIMIZED)
   {
    Minimized = true;
    Hidden = true;
    ShowWindow(hWnd, SW_HIDE); // Hide our window just after the minimize is done.
    Shell_NotifyIcon(NIM_ADD, &NID); //Add the systray icon.
   }
  }break;
  case WM_SYSCOMMAND:
  {
   switch(LOWORD(wParam)) //We capture the 'X' button :)
   {
    case SC_CLOSE:
    {
     ShowWindow(hWnd, SW_HIDE);
    }return(0);
   }
  }break;
  case WM_DESTROY:
  {
   PostQuitMessage(0);
  }break;
 }
 if (MSG == NID.uCallbackMessage) //This is where we get our SysTray Icon notifications.
 {
  switch(lParam)
  {
   case WM_RBUTTONUP: //Did the user right click?
   {//Pop up the menu.
    POINT mp;
    GetCursorPos(&mp);
    TrackPopupMenu(GetSubMenu(hMenu, 0), 0, mp.x, mp.y, 0, hWnd, 0);
   }break;
   case WM_LBUTTONDBLCLK: //Did the user left click?
   {//Show the window and remove the systray icon.
    if (Hidden)
    {
     Hidden = false;
     ShowWindow(hWnd, SW_SHOW);
     if (Minimized)
     {
      Minimized = false;
      ShowWindow(hWnd, SW_RESTORE);
     }
     SetForegroundWindow(hWnd);
    }
    else
    {
     Hidden = true;
     ShowWindow(hWnd, SW_HIDE);
    }
   }break;
  }
 }
 return(DefWindowProc(hWnd, MSG, wParam, lParam));
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
 MSG msg;
 WNDCLASSEX wcx;
 wcx.cbSize = sizeof(wcx);
 wcx.hIconSm = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
 wcx.cbClsExtra = NULL;
 wcx.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
 wcx.lpfnWndProc = ParenWindProc;
 wcx.cbClsExtra = 0;
 wcx.cbWndExtra = 0;
 wcx.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
 wcx.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
 wcx.hCursor = LoadCursor(NULL, IDC_ARROW); 
 wcx.lpszClassName = WinClassName;
 wcx.lpszMenuName = NULL;
 wcx.hInstance = hInstance;
 if (!RegisterClassEx(&wcx)) return(0);
 if (!(ParenthWnd = CreateWindowEx(WS_EX_TOOLWINDOW, WinClassName, "SysTrayIcon", WS_OVERLAPPEDWINDOW, 0, 0, GetSystemMetrics(SM_CXFULLSCREEN), GetSystemMetrics(SM_CYFULLSCREEN), NULL, NULL, hInstance, NULL))) return(0);
 if (!(hWnd = CreateWindowEx(0, WinClassName, "SysTrayIcon", WS_OVERLAPPEDWINDOW, 0, 0, GetSystemMetrics(SM_CXFULLSCREEN), GetSystemMetrics(SM_CYFULLSCREEN), ParenthWnd, NULL, hInstance, NULL))) return(0);

 NID.cbSize = sizeof(NID); // Size of the structure duh :)
 NID.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1)); // The icon that will be shown in the systray.
 NID.uCallbackMessage = WM_USER + 1; // Our user defined message.(For notifications, Windows will send us messages when user pressed on our icon)
 NID.hWnd = hWnd; // Handle to the window we would like to get the notifications.
 NID.uID = WM_USER + 2; // Icon ID in systray...
 strcpy(NID.szTip, "System Tray Icon[QSoft]"); // The tooltip string
 NID.uFlags = NIF_TIP | NIF_ICON | NIF_MESSAGE; // Here we say we want to get notifications about the icon, we have an icon and a tooltip.
 Shell_NotifyIcon(NIM_ADD, &NID);

 ShowWindow(hWnd, SW_SHOW);
 UpdateWindow(hWnd);

 hMenu = LoadMenu(hInstance, "menu1");

 while (1)
	{
  if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
  {
   if (msg.message == WM_QUIT) break;
   TranslateMessage(&msg); 
   DispatchMessage(&msg); 
  }
 }
 Shell_NotifyIcon(NIM_DELETE, &NID);
 return(msg.wParam);
}